package myUIDelegateExample;

import javax.swing.AbstractListModel;
import javax.swing.DefaultListModel;
import javax.swing.event.ListDataListener;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import myUIDelegateExample.Datum;

import java.io.*;
import java.util.*;

/**
 * The WorldRecordsDataModel implements a model of the data that describes the
 * world records, which consists of a number of events, each of which has an
 * event name, a time, a record holder (the person who achieved that time), a
 * nation (the country of the person who established the record), a place (where
 * the record was established), and a date.
 */
public class MyDataModel3 implements TableModel {

	private Vector v;
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	// Recall that the DefaultTableModel stores the rows of the table as a
	// vector.
	// Each element of the vector is a vector, the elements of which are the
	// row's elements for
	// each of the defined columns.
	DefaultTableModel tableModel;

	private static final int REQD_NUM_FIELDS = 6;
	private static final int COUNTRY_FIELD_NUM = 3;

	/**
	 * presently, this constructor creates an instance of a
	 * WorldRecordsDataModel that is populated with data contained in the passed
	 * file, of which the first row provide the header options, and the tokens
	 * are delimited by commas
	 * 
	 * @throws IOException
	 */
	public MyDataModel3(String fileName) {
		List<Datum> list = null;
		try {
			Datum.isWellFormedInputFile(fileName, REQD_NUM_FIELDS);
			list = Datum.parseFile(fileName);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		}

		v = new Vector();
		for (Datum wr : list) {
			v.add(wr.toVector());
		}
		tableModel = new DefaultTableModel(v, Datum.getVectorOfFieldNames());
	}

	public Class<?> getColumnClass(int arg0) {
		Class<?> x = null;
		try {
			x = Class.forName("java.lang.String");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			System.exit(0);
		}
		return x;
	}

	public void removeTableModelListener(TableModelListener arg0) {
		tableModel.removeTableModelListener(arg0);
	}

	public void addTableModelListener(TableModelListener l) {
		System.out.println("adding TableModelListener: " + l);
		tableModel.addTableModelListener(l);
	}

	/**
	 * The body of this method is empty in the parent class AbstractTableModel.
	 * We need to define it.
	 * 
	 */
	public void setValueAt(Object value, int row, int col) {
		System.out.println("setValueAt invoked");
		tableModel.setValueAt(value, row, col);
		// who has invoked me?
		try {
			throw new Exception();
		} catch (Exception e) {
			StackTraceElement[] elements = e.getStackTrace();
			String s = "";
			for (int i = 0; i < elements.length; i++) {
				s += "\t" + elements[i] + "\n";
			}
			System.out.println("Who called the method \"setValueAt\"? ");
			System.out.println(s);
		}
		/*
		 * the tableModel invokes the fireTableCellUpdated method when a value
		 * changes, which in turn dispatches a TableModelEvent and whichever
		 * registered listeners will process the event. One of them is JTable,
		 * which responds by re-rendering the table.
		 * 
		 */
		// System.out.println("fireTableCellUpdated");
	}

	public Object getValueAt(int row, int col) {
		return tableModel.getValueAt(row, col);
	}

	public int getColumnCount() {
		return tableModel.getColumnCount();
	}

	public int getRowCount() {
		return tableModel.getRowCount();
	}

	public boolean isCellEditable(int row, int col) {
		return tableModel.isCellEditable(row, col);
	}

	public String getColumnName(int idx) {
		return tableModel.getColumnName(idx);
	}

}