package myUIDelegateExample;

import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import java.io.*;
import java.util.*;

/**
 * The class implements a model of the data that describes a set of world
 * records, which consists of a number of events, each of which has an event
 * name, a time, a record holder (the person who achieved that time), a nation
 * (the country of the person who established the record), a place (where the
 * record was established), and a date.
 */
public class MyDataModelForTableModel implements TableModel {

	private Vector<Vector<String>> dataAsAVectorOfVectors;
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	// Recall that the DefaultTableModel stores the rows of the table as a
	// vector.
	// Each element of the vector is a vector, the elements of which are the
	// row's elements for
	// each of the defined columns.
	DefaultTableModel wrappedTableModel;

	private static final int REQD_NUM_FIELDS = 6;

	/**
	 * presently, this constructor creates an instance that is populated with
	 * data contained in the passed file, of which the first row provide the
	 * header options, and the tokens are delimited by commas
	 * 
	 * @throws IOException
	 */
	public MyDataModelForTableModel(String fileName) {
		List<Datum> list = null;
		try {
			Datum.isWellFormedInputFile(fileName, REQD_NUM_FIELDS);
			list = Datum.parseFile(fileName);
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(0);
		}

		dataAsAVectorOfVectors = new Vector<Vector<String>>();
		for (Datum wr : list) {
			dataAsAVectorOfVectors.add(wr.toVector());
		}
		wrappedTableModel = new DefaultTableModel(dataAsAVectorOfVectors, Datum
				.getVectorOfFieldNames());
	}

	public Class<?> getColumnClass(int arg0) {
		Class<?> x = null;
		try {
			x = Class.forName("java.lang.String");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			System.exit(0);
		}
		return x;
	}

	public void removeTableModelListener(TableModelListener arg0) {
		wrappedTableModel.removeTableModelListener(arg0);
	}

	public void addTableModelListener(TableModelListener l) {
		System.out.println("adding TableModelListener: " + l);
		wrappedTableModel.addTableModelListener(l);
	}

	/**
	 * The body of this method is empty in the parent class AbstractTableModel.
	 * We need to define it.
	 * 
	 */
	public void setValueAt(Object value, int row, int col) {
		System.out.println("setValueAt invoked");
		wrappedTableModel.setValueAt(value, row, col);
		// who has invoked me?
		try {
			throw new Exception();
		} catch (Exception e) {
			StackTraceElement[] elements = e.getStackTrace();
			String s = "";
			for (int i = elements.length - 1; i >= 0; i--) {
				// for (int i = 0; i < elements.length; i++) {
				s += "\t" + elements[i] + "\n";
			}
			System.out.println("Who called the method \"setValueAt\"? ");
			System.out.println(s);
		}
		/*
		 * the tableModel invokes the fireTableCellUpdated method when a value
		 * changes, which in turn dispatches a TableModelEvent and whichever
		 * registered listeners will process the event. One of them is JTable,
		 * which responds by re-rendering the table.
		 * 
		 */
		// System.out.println("fireTableCellUpdated");
	}

	public Object getValueAt(int row, int col) {
		return wrappedTableModel.getValueAt(row, col);
	}

	public int getColumnCount() {
		return wrappedTableModel.getColumnCount();
	}

	public int getRowCount() {
		return wrappedTableModel.getRowCount();
	}

	public boolean isCellEditable(int row, int col) {
		return wrappedTableModel.isCellEditable(row, col);
	}

	public String getColumnName(int idx) {
		return wrappedTableModel.getColumnName(idx);
	}

}